/*----------------------------------------------------------------------
  File    : JSkel.java
  Contents: Java program skeleton (swing classes)
  Author  : Christian Borgelt
  History : 27.10.2001 file created
            30.10.2001 first version completed
            11.05.2004 scroll pane viewport added
----------------------------------------------------------------------*/
import javax.swing.*;
import javax.swing.filechooser.*;
import java.awt.*;
import java.awt.event.*;

/*--------------------------------------------------------------------*/
class LogoPanel extends JPanel {
/*--------------------------------------------------------------------*/

private Image logo = Toolkit.getDefaultToolkit().getImage("logo.gif");

  public Dimension getPreferredSize ()
  { return new Dimension(100, 101); }  /* size of the logo */

  public void paint (Graphics g)
  { g.setColor(Color.lightGray);
    g.fillRect(0, 0, 100, 101);
    g.drawImage(logo, 0, 0, this);     /* draw the logo */
  }  /* paint() */

}  /* class LogoPanel */


/*--------------------------------------------------------------------*/
class JSkelPanel extends JPanel implements MouseListener {
/*--------------------------------------------------------------------*/

  public void mousePressed (MouseEvent e) {
    Graphics g = getGraphics();
    switch (e.getModifiers()) { /* depending on the mouse button */
      case InputEvent.BUTTON1_MASK: g.setColor(Color.red);   break;
      case InputEvent.BUTTON2_MASK: g.setColor(Color.green); break;
      case InputEvent.BUTTON3_MASK: g.setColor(Color.blue);  break;
      default: return;          /* set the drawing color */
    }                           /* and draw a small circle */
    g.fillOval(e.getX()-3, e.getY()-3, 7, 7);
  }  /* mousePressed() */

  public void mouseReleased (MouseEvent e) { }
  public void mouseClicked  (MouseEvent e) { }
  public void mouseEntered  (MouseEvent e) { }
  public void mouseExited   (MouseEvent e) { }

  /*------------------------------------------------------------------*/

  public void paint (Graphics g)
  {                             /* --- (re)paint the whole panel */
    int       x, y, i;          /* coordinates, loop variable */
    Dimension d = getSize();    /* extensions of the panel */
    String    msg = "This is a test.";

    g.setColor(Color.white);    /* background */
    g.fillRect(0, 0, d.width, d.height);

    g.setColor(Color.red);      /* filled colored circles */
    g.fillOval( 40,  30, 21, 21);
    g.setColor(Color.blue);
    g.fillOval( 70,  60, 21, 21);
    g.setColor(Color.green);
    g.fillOval(100,  90, 21, 21);
    g.setColor(Color.yellow);
    g.fillOval(130, 120, 21, 21);

    g.setColor(Color.red);      /* concentric circles */
    g.drawOval(130,  30, 20, 20);
    g.drawOval(133,  33, 14, 14);
    g.drawOval(136,  36,  8,  8);
    g.setColor(Color.blue);
    g.drawOval(160,  60, 20, 20);
    g.drawOval(163,  63, 14, 14);
    g.drawOval(166,  66,  8,  8);
    g.setColor(Color.green);
    g.drawOval(190,  90, 20, 20);
    g.drawOval(193,  93, 14, 14);
    g.drawOval(196,  96,  8,  8);
    g.setColor(Color.yellow);
    g.drawOval(220, 120, 20, 20);
    g.drawOval(223, 123, 14, 14);
    g.drawOval(226, 126,  8,  8);

    g.setColor(Color.black);    /* tilted black frame */
    g.drawLine( 10,  25, 135, 150);
    g.drawLine( 60,  25, 185, 150);
    g.drawLine( 10,  25,  60,  25);
    g.drawLine(135, 150, 185, 150);

    g.setColor(Color.lightGray);/* speckled area */
    for (i = 0; i < 1000; i++) {
      x = (int)(Math.random() *50);
      y = (int)(Math.random() *50);
      g.fillRect(x +250, y +25, 1, 1);
    }
  
    g.setColor(Color.black);    /* test message */
    i = g.getFontMetrics().stringWidth(msg);
    g.drawString(msg, 160 -i/2, 200);
  }  /* paint() */

}  /* class JSkelPanel */


/*--------------------------------------------------------------------*/
public class JSkel extends JFrame {
/*--------------------------------------------------------------------*/

  private boolean      isApplet;        /* whether started as applet */
  private JFrame       frame   = null;  /* the frame window */
  private JScrollPane  scroll  = null;  /* scroll pane viewport */
  private JSkelPanel   panel   = null;  /* the main panel to draw on */
  private JTextField   stat    = null;  /* a status bar for messages */
  private JFileChooser chooser = null;  /* the file chooser */
  private JDialog      dialog  = null;  /* the example dialog box */
  private JDialog      about   = null;  /* the "About..." dialog box */

  /*------------------------------------------------------------------*/

  private JFileChooser createChooser ()
  {                             /* --- create a file chooser */
    JFileChooser fc = new JFileChooser();
    fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
    fc.setFileHidingEnabled(true);
    fc.setAcceptAllFileFilterUsed(true);
    fc.setMultiSelectionEnabled(false);
    fc.setFileView(null);       /* create a standard file chooser */
    return fc;                  /* without customized filters */
  }  /* createChooser() */

  /*------------------------------------------------------------------*/

  private JDialog createDialog (Component owner)
  {                             /* --- create example dialog */
    final JDialog      dlg  = new JDialog();
    Font               font = new Font("Dialog", Font.BOLD, 12);
    GridBagLayout      g    = new GridBagLayout();
    GridBagConstraints lc   = new GridBagConstraints();
    GridBagConstraints rc   = new GridBagConstraints();
    JPanel             grid = new JPanel(g);
    JPanel             bbar = new JPanel(new GridLayout(1, 2, 5, 5));
    JLabel             lbl;
    JButton            btn;

    grid.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    lc.fill      =              /* fill fields in both directions */
    rc.fill      = GridBagConstraints.BOTH;
    rc.weightx   = 1.0;         /* resize only the input fields, */
    lc.weightx   = 0.0;         /* but not the labels */
    lc.weighty   = 1.0;         /* resize lines equally */
    rc.weighty   = 1.0;         /* in vertical direction */
    lc.ipadx     = 10;          /* gap between labels and inputs */
    lc.ipady     = 10;          /* make all lines of the same height */
    rc.gridwidth = GridBagConstraints.REMAINDER;

    lbl = new JLabel("Give me a text:");
    g.setConstraints(lbl, lc);   grid.add(lbl);
    final JTextField txt = new JTextField("abc");
    txt.setFont(font);
    g.setConstraints(txt, rc);   grid.add(txt);

    lbl = new JLabel("Give me a number:");
    g.setConstraints(lbl, lc);   grid.add(lbl);
    final JTextField num = new JTextField("123");
    num.setFont(font);
    g.setConstraints(num, rc);   grid.add(num);
  
    lbl = new JLabel("Select something:");
    g.setConstraints(lbl, lc);   grid.add(lbl);
    final JComboBox combo = new JComboBox(new String[]
      { "first", "second", "third", "last" });
    g.setConstraints(combo, rc); grid.add(combo);

    lbl = new JLabel("Check me:");
    g.setConstraints(lbl, lc);   grid.add(lbl);
    final JCheckBox check = new JCheckBox();
    g.setConstraints(check, rc); grid.add(check);

    if (!isApplet) {            /* applets cannot access files */
      lbl = new JLabel("Enter a filename:");
      g.setConstraints(lbl, lc);   grid.add(lbl);
      final JTextField fname = new JTextField("noname.dat");
      fname.setFont(font);
      btn = new JButton("Browse...");
      g.setConstraints(btn, rc);   grid.add(btn);
      btn.addActionListener(new AbstractAction () {
        public void actionPerformed (ActionEvent e) {
          if (chooser == null) chooser = createChooser();
          chooser.setDialogType(JFileChooser.OPEN_DIALOG);
          int r = chooser.showDialog(dlg, null);
          if (r == JFileChooser.APPROVE_OPTION)
            fname.setText(chooser.getSelectedFile().getPath());
        } } );
      rc.ipady = 6;             /* adapt the height */
      g.setConstraints(fname, rc); grid.add(fname);
    }

    bbar.setBorder(BorderFactory.createEmptyBorder(5, 0, 0, 0));
    btn = new JButton("Ok"); bbar.add(btn);
    btn.addActionListener(new AbstractAction () {
      public void actionPerformed (ActionEvent e) {
        dlg.setVisible(false);
        System.out.println("\nresult of dialog:");
        System.out.println("text  : " +txt.getText());
        System.out.println("number: " +num.getText());
        System.out.println("select: " +combo.getSelectedIndex());
        System.out.println("flag  : " +check.isSelected());
      } } );
    btn = new JButton("Cancel"); bbar.add(btn);
    btn.addActionListener(new AbstractAction () {
      public void actionPerformed (ActionEvent e) {
        dlg.setVisible(false); } } );
    rc.ipady = 0;               /* put the 'Ok' and the 'Cancel' */
    g.setConstraints(bbar, rc); /* button into a grid layout to make */
    grid.add(bbar);             /* sure that they have the same size */

    dlg.getContentPane().add(grid);
    dlg.setTitle("Dialog...");
    dlg.setModal(true);
    dlg.setLocationRelativeTo(owner);
    dlg.pack();
    return dlg;
  }  /* createDialog() */

  /*------------------------------------------------------------------*/

  private JDialog createAbout (Component owner)
  {                             /* --- create "About..." dialog box */
    final JDialog dlg  = new JDialog();
    Container     pane = dlg.getContentPane();
    LogoPanel     logo = new LogoPanel();
    JButton       btn  = new JButton("Ok");
    JPanel        rest = new JPanel(new BorderLayout(2, 2));
    JTextArea     text = new JTextArea
      ("JSkel\n\n"
      +"A Java Program Skeleton\n"
      +"Version 1.1, 11.05.2004\n\n"
      +"written by Christian Borgelt\n"
      +"Otto-von-Guericke-University of Magdeburg\n"
      +"Universittsplatz 2, D-39106 Magdeburg\n"
      +"e-mail: borgelt@iws.cs.uni-magdeburg.de");
    text.setBackground(Color.lightGray);
    text.setFont(new Font("Dialog", Font.BOLD, 12));
    text.setEditable(false);
    btn.addActionListener(new AbstractAction () {
      public void actionPerformed (ActionEvent e) {
        dlg.setVisible(false); } } );
    rest.setBackground(Color.lightGray);
    rest.add(logo, BorderLayout.NORTH);
    rest.add(btn,  BorderLayout.SOUTH);
    pane.setLayout(new FlowLayout());
    pane.setBackground(Color.lightGray);
    pane.add(text);
    pane.add(rest);
    dlg.setTitle("About JSkel...");
    dlg.setModal(true);
    dlg.setLocationRelativeTo(owner);
    dlg.pack();
    dlg.setResizable(false);
    return dlg;
  }  /* createAbout() */

  /*------------------------------------------------------------------*/

  public JSkel (boolean isApplet)
  {                             /* --- constructor of JSkel class */
    JMenuBar  mbar;             /* menu bar */
    JMenu     menu;             /* to create menu titles */
    JMenuItem item;             /* to create menu items */

    this.isApplet = isApplet;   /* note applet indicator */
    this.getContentPane().setLayout(new BorderLayout());

    /* --- create and set the menu bar --- */
    mbar = new JMenuBar();
    this.getContentPane().add(mbar, BorderLayout.NORTH);

    if (!isApplet) {            /* only applications have a file menu */
      menu = mbar.add(new JMenu("File"));
      menu.setMnemonic('f');
      item = menu.add(new JMenuItem("Load File..."));
      item.setMnemonic('l');
      item.addActionListener(new AbstractAction() {
        public void actionPerformed (ActionEvent e) {
          if (chooser == null) chooser = createChooser();
          chooser.setDialogType(JFileChooser.OPEN_DIALOG);
          int r = chooser.showDialog(JSkel.this, null);
          if (r == JFileChooser.APPROVE_OPTION)
            System.out.println(chooser.getSelectedFile().getPath());
        } });
      item = menu.add(new JMenuItem("Save File..."));
      item.setMnemonic('s');
      item.addActionListener(new AbstractAction() {
        public void actionPerformed (ActionEvent e) {
          if (chooser == null) chooser = createChooser();
          chooser.setDialogType(JFileChooser.SAVE_DIALOG);
          int r = chooser.showDialog(JSkel.this, null);
          if (r == JFileChooser.APPROVE_OPTION)
            System.out.println(chooser.getSelectedFile().getPath());
        } });
      menu.addSeparator();
      item = menu.add(new JMenuItem("Quit"));
      item.setMnemonic('q');
      item.addActionListener(new AbstractAction() {
        public void actionPerformed (ActionEvent e) {
          System.exit(0); } });
    }

    menu = mbar.add(new JMenu("Actions"));
    menu.setMnemonic('a');
    item = menu.add(new JMenuItem("Dialog..."));
    item.setMnemonic('d');
    item.addActionListener(new AbstractAction() {
      public void actionPerformed (ActionEvent e) {
        if (dialog == null) dialog = createDialog(JSkel.this);
        dialog.setVisible(true);
      } });
    item = menu.add(new JMenuItem("Redraw"));
    item.setMnemonic('r');
    item.addActionListener(new AbstractAction() {
      public void actionPerformed (ActionEvent e) {
        panel.repaint(); } });

    menu = mbar.add(new JMenu("Help"));
    menu.setMnemonic('h');
    item = menu.add(new JMenuItem("About..."));
    item.setMnemonic('a');
    item.addActionListener(new AbstractAction() {
      public void actionPerformed (ActionEvent e) {
        if (about == null) about = createAbout(JSkel.this);
        about.setVisible(true);
      } });

    /* --- create and set the main panel --- */
    panel = new JSkelPanel(); 
    panel.setLayout(new BorderLayout());
    panel.setPreferredSize(new Dimension(320, 240));
    panel.addMouseListener(panel);
    scroll = new JScrollPane(panel);
    this.getContentPane().add(scroll, BorderLayout.CENTER);

    /* --- create and set a status bar --- */
    /*
    stat = new JTextField("");
    stat.setEditable(false);
    this.getContentPane().add(stat, BorderLayout.SOUTH);
    */

    if (isApplet) return;       /* if called as an applet, abort */

    /* --- create and show the frame window --- */
    this.setTitle("JSkel");
    this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    this.setLocation(50,50);
    this.pack(); this.setVisible(true);
  }  /* JSkel() */

  /*------------------------------------------------------------------*/

  public static void main (String args[])
  {                             /* --- main function */
    JSkel s = new JSkel(false); /* (called only if this class is */
  }  /* main() */               /*  executed as an application) */

}  /* class JSkel */
